@@ -4,8 +4,8 @@
 ## Author:      Mattia Barbon
 ## Modified by:
 ## Created:     11/02/2001
-## RCS-ID:      $Id: wxTextEntryDialog.pm 2189 2007-08-21 18:15:31Z mbarbon $
-## Copyright:   (c) 2001, 2003, 2006 Mattia Barbon
+## RCS-ID:      $Id: wxTextEntryDialog.pm 2212 2007-08-25 23:13:10Z mbarbon $
+## Copyright:   (c) 2001, 2003, 2006-2007 Mattia Barbon
 ## Licence:     This program is free software; you can redistribute it and/or
 ##              modify it under the same terms as Perl itself
 #############################################################################
@@ -29,7 +29,7 @@ sub commands {
 sub text_entry_dialog {
   my( $this ) = @_;
   my $dialog = Wx::TextEntryDialog->new
-    ( $this, "Enter som text", "Wx::TextEntryDialog sample",
+    ( $this, "Enter some text", "Wx::TextEntryDialog sample",
       "I am a default value" );
 
   if( $dialog->ShowModal == wxID_CANCEL ) {
